@@ -13,11 +13,31 @@
 #    under the License.
 
 import collections
+import logging
 import os
 import sys
 import time
 import uuid
 
+# custom_index.py added in ovs 2.10. We just need the IndexEntryClass function
+# which is copied from OVS below since the current lower constraint is < 2.10
+try:
+    from ovs.db.custom_index import IndexEntryClass
+except ImportError:
+    from ovs.db import data
+
+    def IndexEntryClass(table):
+        def defaults_uuid_to_row(atom, base):
+            return atom.value
+
+        columns = ['uuid'] + list(table.columns.keys())
+        cls = collections.namedtuple(table.name, columns)
+        cls._table = table
+        cls.__new__.__defaults__ = (None,) + tuple(
+            data.Datum.default(c.type).to_python(defaults_uuid_to_row)
+            for c in table.columns.values())
+        return cls
+
 from ovs.db import idl
 from ovs import jsonrpc
 from ovs import poller
@@ -27,6 +47,7 @@ import six
 from ovsdbapp import api
 from ovsdbapp import exceptions
 
+LOG = logging.getLogger(__name__)
 
 RowLookup = collections.namedtuple('RowLookup',
                                    ['table', 'column', 'uuid_column'])
@@ -114,21 +135,48 @@ def get_schema_helper(connection, schema_name):
     :param schema_name: The schema on the server to pull
     :type schema_name: string
     """
-    err, strm = stream.Stream.open_block(
-        stream.Stream.open(connection))
-    if err:
-        raise Exception("Could not connect to %s" % connection)
-    rpc = jsonrpc.Connection(strm)
-    req = jsonrpc.Message.create_request('get_schema', [schema_name])
-    err, resp = rpc.transact_block(req)
-    rpc.close()
-    if err:
-        raise Exception("Could not retrieve schema from %(conn)s: "
-                        "%(err)s" % {'conn': connection,
-                                     'err': os.strerror(err)})
-    elif resp.error:
-        raise Exception(resp.error)
-    return idl.SchemaHelper(None, resp.result)
+    parsed_connections = parse_connection(connection)
+
+    for c in parsed_connections:
+        err, strm = stream.Stream.open_block(
+            stream.Stream.open(c))
+        if err:
+            LOG.error("Unable to open stream to %(conn)s to retrieve schema: "
+                      "%(err)s", {'conn': c,
+                                  'err': os.strerror(err)})
+            continue
+        rpc = jsonrpc.Connection(strm)
+        req = jsonrpc.Message.create_request('get_schema', [schema_name])
+        err, resp = rpc.transact_block(req)
+        rpc.close()
+        if err:
+            LOG.info("Could not retrieve schema from %(conn)s: "
+                     "%(err)s", {'conn': c,
+                                 'err': os.strerror(err)})
+            continue
+        elif resp.error:
+            LOG.error("TRXN error, failed to retrieve schema from %(conn)s: "
+                      "%(err)s", {'conn': c,
+                                  'err': resp.error})
+            continue
+        return idl.SchemaHelper(None, resp.result)
+    raise Exception("Could not retrieve schema from %s" % connection)
+
+
+def parse_connection(connection_string):
+    """Parse a connection string.
+
+    The connection string must be of the form
+    proto:address:port,proto:address:port,...
+
+    The parsing logic here must be identical to the one at
+    https://github.com/openvswitch/ovs/blob/master/python/ovs/db/idl.py#L162
+    for remote connections.
+
+    :param connection_string: The ovsdb-server connection string
+    :type connection_string: string
+    """
+    return [c.strip() for c in connection_string.split(',')]
 
 
 def wait_for_change(_idl, timeout, seqno=None):
@@ -156,7 +204,7 @@ def get_column_value(row, col):
         val = getattr(row, col)
 
     # Idl returns lists of Rows where ovs-vsctl returns lists of UUIDs
-    if isinstance(val, list) and len(val):
+    if isinstance(val, list) and val:
         if isinstance(val[0], idl.Row):
             val = [v.uuid for v in val]
         col_type = row._table.columns[col].type
@@ -177,7 +225,11 @@ def condition_match(row, condition):
 
     # both match and val are primitive types, so type can be used for type
     # equality here.
-    if type(match) is not type(val):
+    # NOTE (twilson) the above is a lie--not all string types are the same
+    #                I haven't investigated the reason for the patch that
+    #                added this code, but for now I check string_types
+    if type(match) is not type(val) and not all(
+        isinstance(x, six.string_types) for x in (match, val)):
         # Types of 'val' and 'match' arguments MUST match in all cases with 2
         # exceptions:
         # - 'match' is an empty list and column's type is optional;
@@ -201,7 +253,7 @@ def condition_match(row, condition):
     if isinstance(match, dict):
         for key in match:
             if op == '=':
-                if (key not in val or match[key] != val[key]):
+                if key not in val or match[key] != val[key]:
                     matched = False
                     break
             elif op == '!=':
@@ -277,3 +329,34 @@ def db_replace_record(obj):
     elif isinstance(obj, api.Command):
         obj = obj.result
     return obj
+
+
+def row2str(row):
+    """Get a string representation of a Row"""
+
+    # This is not a repr, as the Row object takes a dict of Datum objects and
+    # we don't really want to deal with those, just what the Python values are.
+    # Row foreign keys are printed as their UUID
+    return "%s(%s)" % (row._table.name, ", ".join(
+        "%s=%s" % (col, idl._row_to_uuid(getattr(row, col)))
+        for col in row._table.columns if hasattr(row, col)))
+
+
+def frozen_row(row):
+    """Return a namedtuple representation of a idl.Row object
+
+    Row objects are inherently tied to the transaction processing of the Idl.
+    This means that if you have a reference to a Row in one thread, and
+    another thread starts a transaction that modifies that row, the Row can
+    change w/o you knowing it. This is especially noticeable when using the
+    RowEventHandler. It is possible for a Row that is passed to notify() by
+    the Idl class to change between being matched and the RowEvent.run()
+    method being called. This returns an immutable representation of the row
+    by using the same class that custom indexes use for searching. This
+    should be safe to pass to other threads.
+    """
+    IndexEntry = IndexEntryClass(row._table)
+    return IndexEntry(
+        uuid=row.uuid,
+        **{col: getattr(row, col)
+           for col in row._table.columns if hasattr(row, col)})
